﻿using UnityEngine;
using System.Collections;

public class LevelGenerator : MonoBehaviour {
	public Transform groundPrefab;
	public Transform level1Prefab;
	public Transform level2Prefab;
	public Transform groundCoinPrefab;
	public Transform level1CoinPrefab;
	public Transform level2CoinPrefab;
	public Transform groundMonsterPrefab;
	public Transform level1MonsterPrefab;
	public Transform level2MonsterPrefab;
	public Transform groundBoxPrefab;
	public Transform level1BoxPrefab;
	public Transform level2BoxPrefab;

	void Start () {
		// 50 talaj szélességnyi lesz a pálya
		int levelSize = 50;
		// minden 6. talajból szakadék lesz
		float groundProb = 5.0f / 6.0f;
		// minden második talaj fölött lesz egy emelet
		float level1Prob = 0.5f;
		// minden második emelet fölött lesz még egy emelet
		float level2Prob = 0.5f;
		// minden harmadik talajon/emeleten lesz érme
		float coinProb = 1.0f/3.0f;
		// minden negyedik talajon/emeleten lesz szörny
		float monsterProb = 0.25f;
		// ha sem érme, sem szörny nem volt, akkor 50% eséllyel lerakunk egy dobozt
		float boxProb = 0.5f;
		// segédváltozó: mikor raktunk le talajt utoljára. A -10 azt jelenti elég régen
		int lastGround = -10;
		// segédváltozó: mikor raktunk le emeletet utoljára. A -10 azt jelenti elég régen
		int lastLevel1 = -10;
		// végigmegyünk a pályaelemeken 50 db + 3 sima tlaj bevezetés és 3 sima talajelem lezárásnak (ide lehet majd start/end feliratot rakni)
		for (int i = -3; i < levelSize+3; ++i) {
			// hova kerüljön az új pályaelem. Világtérbeli eltolásvektor az origóhoz képest. Csak az x koordinátát határozza meg, az y a prefabokban már be lett állítva
			Vector3 offset = new Vector3(i*10,0,0);
			// az első és az utolsó 3 elem egyszerű talaj
			if(i < 0 || i >= levelSize)
			{
				//példányosítás (talaj elem)
				Transform newGroundElement = Instantiate<Transform> (groundPrefab);
				// x irányban helyére toljuk
				newGroundElement.Translate(offset);
			}
			else
			{
				//ground				
				bool ground = false; //segédváltozó: leraktunk-e talajelemet
				float rand = Random.value; //kérünk egy új véletlen számot 0 és 1 között
				if(rand < groundProb //ha a véletlen szám teljesíti a valószínűséget
				 	|| (lastGround < i-1  // ha mégsem teljesítené, akkor szakadék kell, de nem lehet egymás mellett két szakadék (utoljára lerakott talaj elem nem az előző, hanem régebben történt), ilyenkor le kell raknunk talajt					
						&& lastLevel1 < i-2) //de ezt még annyiban finomítjuk, hogy ha volt emelet az előző előtti lerakásban, akkor még át tudnánk ugrani, tehát csak akkor rakjunk talajt, ha emeletet ennél régebben raktunk le.
					)
				{
					//talaj kell, tehát pédányosítsunk
					Transform newGroundElement = Instantiate<Transform> (groundPrefab);
					// x irányban helyére toljuk
					newGroundElement.Translate(offset);
					lastGround = i;//segédváltozó, fentebb láttuk a használatát
					ground = true;//leraktunk talajt

					bool coin = false; //talaj szinten raktunk-e le érmét
					rand = Random.value; //új véletlen szám
					if(rand < coinProb){
						//Ha a véletlen számmal azt sorsoltuk, hogy érme kell
						//példányosítsunk (talaj szintjén álló érmét)
						Transform newCoinElement = Instantiate<Transform> (groundCoinPrefab);
						// x irányban helyére toljuk
						newCoinElement.Translate(offset);
						coin = true; //raktunk le érmét a talajra
					}
					bool monster = false; // talaj szinten raktunk-e le szörnyet
					rand = Random.value;//új véletlen szám
					if(rand < monsterProb){
						//Ha a véletlen számmal azt sorsoltuk, hogy szörny kell
						//példányosítsunk (talaj szintjén álló szörnyet)
						Transform newMonsterElement = Instantiate<Transform> (groundMonsterPrefab);
						// x irányban helyére toljuk
						newMonsterElement.Translate(offset);
						monster = true;//raktunk le szörnyet a talajra
					}
					if(!coin && !monster) // ha a talajra nem raktunk még se szörnyet, se érmét
					{
						rand = Random.value; //új véletlen szám
						if(rand < boxProb){
							//Ha a véletlen számmal azt sorsoltuk, hogy doboz kell
							//példányosítsunk (talaj szintjén álló dobozt)
							Transform newBoxElement = Instantiate<Transform> (groundBoxPrefab);
							// x irányban helyére toljuk
							newBoxElement.Translate(offset);						
						}
					}
				}
				//Ezzel kész a talaj szint
				//Az első emeletet hasonlóan hozzuk létre
				//first floor
				bool level1 = false;
				if(ground) // csak akkor rakunk le emeletet, ha van alatta talaj
				{
					rand = Random.value; //véletlen szám
					if(rand < level1Prob)
					{
						//példányosítás
						Transform newElement = Instantiate<Transform> (level1Prefab);
						//eltolás
						newElement.Translate(offset);
						//raktunk le 1. emeletet
						level1 = true;
						//ez az utolsó lerakott emelet
						lastLevel1 = i;
						
						//érmét az első emletre ugyanúgy csináljuk, mint a talajra
						bool coin = false;
						rand = Random.value;
						if(rand < coinProb){
							Transform newCoinElement = Instantiate<Transform> (level1CoinPrefab);
							newCoinElement.Translate(offset);
							coin = true;
						}
						//szörnyet az első emletre ugyanúgy csináljuk, mint a talajra
						bool monster = false;
						rand = Random.value;
						if(rand < monsterProb){
							Transform newMonsterElement = Instantiate<Transform> (level1MonsterPrefab);
							newMonsterElement.Translate(offset);
							monster = true;
						}
						//dobozt az első emletre ugyanúgy csináljuk, mint a talajra
						if(!coin && !monster)
						{
							rand = Random.value;
							if(rand < boxProb){
								Transform newBoxElement = Instantiate<Transform> (level1BoxPrefab);
								newBoxElement.Translate(offset);						
							}
						}
					}
				}
				
				//Második emelet az elsőhöz hasonlóan történik
				//second floor
				bool level2 = false;
				if(level1) //csak akkor rakunk le második emeletet, ha volt alatta első emelet
				{
					rand = Random.value;
					if(rand < level2Prob)
					{
						Transform newElement = Instantiate<Transform> (level2Prefab);
						newElement.Translate(offset);
						level2 = true;

						bool coin = false;
						rand = Random.value;
						if(rand < coinProb){
							Transform newCoinElement = Instantiate<Transform> (level2CoinPrefab);
							newCoinElement.Translate(offset);
							coin = true;
						}

						bool monster = false;
						rand = Random.value;
						if(rand < monsterProb){
							Transform newMonsterElement = Instantiate<Transform> (level2MonsterPrefab);
							newMonsterElement.Translate(offset);
							monster = true;
						}

						if(!coin && !monster)
						{
							rand = Random.value;
							if(rand < boxProb){
								Transform newBoxElement = Instantiate<Transform> (level2BoxPrefab);
								newBoxElement.Translate(offset);						
							}
						}
					}
				}
			}
		}
	}
}
